; ****************************************************
; ovlada My602 pro PMD-85 (3rd generation)
; ****************************************************
; vstupn port: 8Ch
; formt bit portu: PL--XXYY
; 6. 7. 2020 - vrazn pravy => 3rd generation
;              13500-14500 ten/sec
; 12. 12. 2020 - oprava dynamiky pohybu dle Pythagora
; ****************************************************

; ******************************************************
; Rutina pro rozpoznn pohybu myi typu 602.
; Detekce pohybu myi bez pekreslovn jejho kurzoru.
; ******************************************************
; Po nvratu je stav L tlatka myi ptomen
; v nejvym bitu reg. A (a je nastaven pznak S)
; a stav pravho tlatka je v pznaku CY.
; Proto: JC adr => skok pi P tlatku
;        JM adr => skok pi L tlatku
; ******************************************************

MysDriver
	xra a
	mov b,a		; BC = prstek souadnice X
	mov c,a
	mov d,a		; DE = prstek souadnice Y
	mov e,a
	dcr a		; odpoet doby chodu
	sta MysRpt	; (ideln je velk slo, max. 255)
			; doba chodu nesm bt vt ne 255,
			; protoe pak neplat nkter
			; pedpoklady u matematickch prav
MysLoop
	in mysport	; nast signly myi
	ani 0fh		; ponechat jen clonky
MysOld	adi 0		; plus star stav clonek
	mov l,a		; index v tabulce pechod
	add a		; star stav se vysune z ACC
	add a		; a nov stav clonek se
	add a		; ulo do vy poloviny bajtu
	add a		; a nsledn do pamti jako
	sta MysOld+1	; star stav clonek
	mvi h,MysTab/256 ; zatek tabulky pechod
	mov l,m		; nast LO-adr ovladae smru
	dcr h		; k nmu HI-adr ovladae smru
			; ve strnce 00xxh
	pchl		; a skok na tento ovlada

; ************************************************
; Obslun vektory pro smry pohybu myi
; mus leet ve 256 bajtov strnce ped
; tabulkou pechodovch stav clonek myi "mystab"
; tabulka "mystab" obsahuje jen LO-adr vektor!!!
; ************************************************

mys_sv:	dcx d		; Y--
mys_vv:	inx b		; X++
mysnul:	lxi h,MysRpt
	dcr m
	jnz MysLoop
	jmp MysQuit

mys_jz:	inx d		; Y++
mys_zz:	dcx b		; X--
	lxi h,MysRpt
	dcr m
	jnz MysLoop
	jmp MysQuit

mys_sz:	dcx b		; X--
mys_ss:	dcx d		; Y--
	lxi h,MysRpt
	dcr m
	jnz MysLoop
	jmp MysQuit

mys_jv:	inx b		; X++
mys_jj:	inx d		; Y++
	lxi h,MysRpt
	dcr m
	jnz MysLoop
	jmp MysQuit

; **********************************************
; Tabulka nich bajt adres obslunch vektor
; (vy bajt adresy obslunho vektoru je vdy!
; o jedniku men ne vy bajt bzov adresy
; tabulky "mystab").
; Tyto equations nepesouvat ped rutiny!!!
; **********************************************

m60	.equ mysnul%256	; nulov funkce
m61	.equ mys_ss%256	; sever
m62	.equ mys_sv%256	; severovchod
m63	.equ mys_vv%256	; vchod
m64	.equ mys_jv%256	; jihovchod
m65	.equ mys_jj%256	; jih
m66	.equ mys_jz%256	; jihozpad
m67	.equ mys_zz%256	; zpad
m68	.equ mys_sz%256	; severozpad

; ***********************************************************
; Bufer pro zlohu pozad kurzoru myi. Mus konit na adrese
; dliteln 256-ti, aby pi inkrementaci LSB generoval Z=1.
; ***********************************************************

	.org ($+23)/256*256+0e8h
 
MysBufer
	.ds 24		; vlastn bufer pozad kurzoru myi

; ***********************************************
; Tabulka pechodovch stav clonek myi 602.
; Obsahuje LO-adr obslunch vektor detekovanch
; smr. Tabulka mus zanat na adrese xx00h!
; ***********************************************

	; ni nibble adresy je nov stav clonek
	; vy nibble adresy je star stav clonek
	; 01/02 v rmci nibble je svisl smr
	; 04/08 v rmci nibble je vodorovn smr

	.org MysBufer+24	; zarovnat na adresu s LSB=00h
				; tabulka pro detekci pohybu

MysTab:	.db m60,m65,m61,m60,m63,m64,m62,m63 ; 00
	.db m67,m66,m68,m67,m60,m65,m61,m60
	.db m61,m60,m60,m65,m62,m63,m63,m64 ; 10
	.db m68,m67,m67,m66,m61,m60,m60,m65
	.db m65,m60,m60,m61,m64,m63,m63,m62 ; 20
	.db m66,m67,m67,m68,m65,m60,m60,m61
	.db m60,m61,m65,m60,m63,m62,m64,m63 ; 30
	.db m67,m68,m66,m67,m60,m61,m65,m60
	.db m67,m66,m68,m67,m60,m65,m61,m60 ; 40
	.db m60,m65,m61,m60,m63,m64,m62,m63
	.db m68,m67,m67,m66,m61,m60,m60,m65 ; 50
	.db m61,m60,m60,m65,m62,m63,m63,m64
	.db m66,m67,m67,m68,m65,m60,m60,m61 ; 60
	.db m65,m60,m60,m61,m64,m63,m63,m62
	.db m67,m68,m66,m67,m60,m61,m65,m60 ; 70
	.db m60,m61,m65,m60,m63,m62,m64,m63
	.db m63,m64,m62,m63,m60,m65,m61,m60 ; 80
	.db m60,m65,m61,m60,m67,m66,m68,m67
	.db m62,m63,m63,m64,m61,m60,m60,m65 ; 90
	.db m61,m60,m60,m65,m68,m67,m67,m66
	.db m64,m63,m63,m62,m65,m60,m60,m61 ; A0
	.db m65,m60,m60,m61,m66,m67,m67,m68
	.db m63,m62,m64,m63,m60,m61,m65,m60 ; B0
	.db m60,m61,m65,m60,m67,m68,m66,m67
	.db m60,m65,m61,m60,m67,m66,m68,m67 ; C0
	.db m63,m64,m62,m63,m60,m65,m61,m60
	.db m61,m60,m60,m65,m68,m67,m67,m66 ; D0
	.db m62,m63,m63,m64,m61,m60,m60,m65
	.db m65,m60,m60,m61,m66,m67,m67,m68 ; E0
	.db m64,m63,m63,m62,m65,m60,m60,m61
	.db m60,m61,m65,m60,m67,m68,m66,m67 ; F0
	.db m63,m62,m64,m63,m60,m61,m65,m60

; ************************************************************
; Tabulka pixelovch ofset v ose X pro vbr kurzoru
; dle pixelov pozice 0..5
; Tabulka vrac hodnotu MOD3 (dvma se peddl instrukc RAR)
; Rozsah tabulky (vstupnch hodnot) je 0..(287/2)
; ************************************************************

	.org ($+255)/256*256	; zarovnat na adresu s LSB=00h

tbPix2Ofs
	.db 0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2
	.db 0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2
	.db 0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2
	.db 0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2
	.db 0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2
	.db 0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2

; ******************************************
; Kontrola rozsahu souadnice X kurzoru myi
;  vstup: HL = souadnice X
; vstup: HL = souadnice X omezen dorazy
; ******************************************

LoadXCoor
	lhld MysX	; souadnici X
TestXCoor
	xchg
	lhld MysXMin	; a L doraz
	rst 4		; porovnm
	rnc		; X < L doraz, vracm L doraz
	
	lhld MysXMax	; natu P doraz
	rst 4		; porovnm
	xchg
	rnc		; X < P doraz, souadnice X je OK

	mov a,h		; kdy je X > P doraz, me bt
	ana a		; jet souadnice X zporn..
	xchg
	rp		; nen => vracm tedy P doraz
	lhld MysXMin	; je => vracm L doraz
	ret

; ******************************************
; Kontrola rozsahu souadnice Y kurzoru myi
;  vstup: HL = souadnice Y
; vstup: HL = souadnice Y omezen dorazy
; ******************************************

LoadYCoor
	lhld MysY	; souadnici Y
TestYCoor
	xchg
	lhld MysYMin	; a H doraz
	rst 4		; porovnm
	rnc		; Y je nad H dorazem, vracm H doraz

	lhld MysYMax	; natu D doraz
	rst 4		; porovnm
	xchg
	rnc		; Y nad D dorazem, souadnice Y je OK

	mov a,h		; kdy je Y pod D dorazem, me bt
	ana a		; jet souadnice Y zporn..
	xchg
	rp		; nen => vracm tedy D doraz
	lhld MysYMin	; je => vracm H doraz
	ret

; *******************************************
; Deaktivace myi
; Obnov pozad pod kurzorem myi z buferu.
; Bufer sm je adresovn linern, ovem
; obsah je zpehzen pro adresaci vram 0-4R-8
; *******************************************

MysDea	lxi b,-(7*64)-1	; BC = dkova
	lxi d,MysBufer	; DE = bufer pozad
	lhld MysAOld

RefreshLoop
	ldax d		; 0
	mov m,a
	inr l
	inr e
	ldax d
	mov m,a
	inr h
	inr e

	ldax d		; 4R
	mov m,a
	dcr l
	inr e
	ldax d
	mov m,a
	stax d
	inr h
	inr e

	ldax d		; 8
	mov m,a
	inr l
	inr e
	ldax d
	mov m,a
	dad b
	inr e

	jnz RefreshLoop	; konec buferu?
	ret

; ********
; promnn
; ********

MysX	.dw 0		; souadnice X kurzoru myi
MysY	.dw 0		; souadnice Y kurzoru myi
MysXMin	.dw 0		; L doraz kurzoru myi (min 0!!!)
MysXMax	.dw 281		; P doraz kurzoru myi (max 281!!!)
MysYMin	.dw 0		; H doraz kurzoru myi (min 0!!!)
MysYMax	.dw 244		; D doraz kurzoru myi (max 244)
MysAdr	.dw 0		; adresa kurzoru myi ve videoram
MysAOld	.dw 0		; star adresa kurzoru myi ve videoram
MysXCHG	.db 0		; pznak zmny pozice kurzoru
MysRpt	.db 0		; poitadlo opakovn ve smyce
MysBit	.dw MysCrv	; ukazatel na vybran kurzor dle MysX

; ***********************************************************
; Tabulka adres v ose X pro posun kurzoru dle pixelov pozice
; Tabulka vrac hodnotu /3 (dvma se peddl instrukc RAR)
; Rozsah tabulky (vstupnch hodnot) je 0..(287/2)
; ***********************************************************

	.org ($+255)/256*256	; zarovnat na adresu s LSB=00h

tbPix2Adr
	.db 00,00,00,01,01,01,02,02,02,03,03,03,04,04,04
	.db 05,05,05,06,06,06,07,07,07,08,08,08,09,09,09
	.db 10,10,10,11,11,11,12,12,12,13,13,13,14,14,14
	.db 15,15,15,16,16,16,17,17,17,18,18,18,19,19,19
	.db 20,20,20,21,21,21,22,22,22,23,23,23,24,24,24
	.db 25,25,25,26,26,26,27,27,27,28,28,28,29,29,29
	.db 30,30,30,31,31,31,32,32,32,33,33,33,34,34,34
	.db 35,35,35,36,36,36,37,37,37,38,38,38,39,39,39
	.db 40,40,40,41,41,41,42,42,42,43,43,43,44,44,44
	.db 45,45,45,46,46,46,47,47,47
	
; *********************
; aktivace myi
; - test validn adresy
; - uschovn pozad
; - vykreslen kurzoru
; *********************

MysAct

; *******************************************
; uschov pozad pod kurzorem myi do buferu
; bufer sm je adresovn linern, ovem
; obsah je zpehzen pro adresaci vram 0-4R-8
; *******************************************

	call Pixel2Adr	; HL = adresa kurzoru myi
			; vetn testu validity
MysActBufer
	lxi b,-(7*64)-1	; BC = dkova
	lxi d,MysBufer	; DE = bufer pozad
MysSaveLoop
	mov a,m		; [0,0]
	stax d
	inr l
	inr e
	mov a,m		; [1,0]
	stax d
	inr h
	inr e

	mov a,m		; [1,4]
	stax d
	dcr l
	inr e
	mov a,m		; [0,4]
	stax d
	inr h
	inr e

	mov a,m		; [0,8]
	stax d
	inr l
	inr e
	mov a,m		; [1,8]
	stax d
	dad b
	inr e

	jnz MysSaveLoop	; konec buferu?

; ******************************
; nakresl kurzor myi na pozad
; ******************************

ShowMys
	lxi h,0         ; uschovat hodnotu SP
	dad sp
	shld ShowMysSP+1

	lhld MysBit	; H:L = 02:uu, kde uu je bitov
			;       pozice kurzoru myi 0..5
	mov l,m         ; H:L = 02:vv, co je pln adresa
			;       s animan fz kurzoru myi
	sphl            ; SP = pedloha kurzoru myi podle
			;      horizontln pozice v bajtu
	lhld MysAdr     ; HL = adresa kurzoru
	shld MysAOld
	lxi d,MysBufer    ; DE = bufer pozad kurzoru

ShowMysLoop
	pop b           ; 0
	ldax d
	inr e
	ana b
	ora c
	mov m,a
	inr l
	pop b
	ldax d
	inr e
	ana b
	ora c
	mov m,a
	inr h

	pop b           ; 4R
	ldax d
	inr e
	ana b
	ora c
	mov m,a
	dcr l
	pop b
	ldax d
	inr e
	ana b
	ora c
	mov m,a
	inr h

	pop b           ; 8
	ldax d
	inr e
	ana b
	ora c
	mov m,a
	inr l
	pop b
	ldax d
	ana b
	ora c
	mov m,a
	lxi b,-(7*64)-1 ; dkova
	dad b

	inr e
	jnz ShowMysLoop

ShowMysSP
	lxi sp,0        ; obnova hodnoty reg. SP
	ret

; ******************************************************
; 6 animanch fz myho kurzoru.
; Tabulka ofset a po nvt myscr5 mus leet v jedn
; 256-bajtov strnce!
; ******************************************************

	.org ($+255)/256*256	; zarovnat na adresu s LSB=00h

MysCrv:	.db myscr0%256  ; ofset fze 0
	.db myscr1%256  ; ofset fze 1
	.db myscr2%256  ; ofset fze 2
	.db myscr3%256  ; ofset fze 3
	.db myscr4%256  ; ofset fze 4
	.db myscr5%256  ; ofset fze 5

myscr0: .dw 0f800h,0ff00h ; 0 - fze 0
	.dw 0fe00h,0c01eh ; 4R
	.dw 0c010h,0fe00h ; 8
	.dw 0f002h,0ff00h ; 1
	.dw 0fe00h,0c03eh ; 5R
	.dw 0c720h,0fe00h ; 9
	.dw 0e006h,0ff00h ; 2
	.dw 0fe00h,0c00eh ; 6R
	.dw 0cf20h,0fe00h ; 10
	.dw 0c00eh,0ff00h ; 3
	.dw 0ff00h,0c012h ; 7R
	.dw 0cf00h,0fe00h ; 11

myscr1:	.dw 0f100h,0ff00h ; 0 - fze 1
	.dw 0fc00h,0c13ch ; 4R
	.dw 0c120h,0fc00h ; 8
	.dw 0e104h,0ff00h ; 1
	.dw 0fc01h,0c13ch ; 5R
	.dw 0cf00h,0fc01h ; 9
	.dw 0c10ch,0ff00h ; 2
	.dw 0fc00h,0c11ch ; 6R
	.dw 0df00h,0fc01h ; 10
	.dw 0c11ch,0fe00h ; 3
	.dw 0fe00h,0c124h ; 7R
	.dw 0df00h,0fc00h ; 11

myscr2:	.dw 0e300h,0ff00h ; 0 - fze 2
	.dw 0f801h,0c338h ; 4R
	.dw 0c300h,0f801h ; 8
	.dw 0c308h,0ff00h ; 1
	.dw 0f803h,0c338h ; 5R
	.dw 0df00h,0f802h ; 9
	.dw 0c318h,0fe00h ; 2
	.dw 0f800h,0c338h ; 6R
	.dw 0ff00h,0f802h ; 10
	.dw 0c338h,0fc00h ; 3
	.dw 0fc01h,0c308h ; 7R
	.dw 0ff00h,0f800h ; 11

myscr3: .dw 0c700h,0ff00h ; 0 - fze 3
	.dw 0f003h,0c730h ; 4R
	.dw 0c700h,0f002h ; 8
	.dw 0c710h,0fe00h ; 1
	.dw 0f007h,0c730h ; 5R
	.dw 0ff00h,0f004h ; 9
	.dw 0c730h,0fc00h ; 2
	.dw 0f001h,0c730h ; 6R
	.dw 0ff00h,0f104h ; 10
	.dw 0c730h,0f801h ; 3
	.dw 0f802h,0c710h ; 7R
	.dw 0ff00h,0f100h ; 11

myscr4: .dw 0cf00h,0fe00h ; 0 - fze 4
	.dw 0e007h,0cf20h ; 4R
	.dw 0cf00h,0e004h ; 8
	.dw 0cf20h,0fc00h ; 1
	.dw 0e00fh,0cf20h ; 5R
	.dw 0ff00h,0e108h ; 9
	.dw 0cf20h,0f801h ; 2
	.dw 0e003h,0cf20h ; 6R
	.dw 0ff00h,0e308h ; 10
	.dw 0cf20h,0f003h ; 3
	.dw 0f004h,0cf20h ; 7R
	.dw 0ff00h,0e300h ; 11

myscr5:	.dw 0df00h,0fc00h ; 0 - fze 5
	.dw 0c00fh,0df00h ; 4R
	.dw 0df00h,0c008h ; 8
	.dw 0df00h,0f801h ; 1
	.dw 0c01fh,0df00h ; 5R
	.dw 0ff00h,0c310h ; 9
	.dw 0df00h,0f003h ; 2
	.dw 0c007h,0df00h ; 6R
	.dw 0ff00h,0c710h ; 10
	.dw 0df00h,0e007h ; 3
	.dw 0e009h,0df00h ; 7R
	.dw 0ff00h,0c700h ; 11

; *********************************************************
; Z pixelovch souadnic, kter oee na platn rozsah,
; vypote adresu kurzoru ve videoram
; vstup: promnn MysX, MysY
; vstup: HL = adresa kurzoru ve videoram
; 
; *********************************************************

Pixel2Adr
	call LoadYCoor	; HL := 11yyyyyy yy000000
	shld MysY
	dad h 
	dad h 
	dad h 
	dad h 
	dad h 
	dad h
	mov a,h
	adi 0c0h
	mov h,a
	push h

	call LoadXCoor	; HL := HL + pozice bajtu na dku
	shld MysX	; tedy HL := 11yyyyyy  yyxxxxxx
	mov a,h
	rar
	mov a,l
	rar
	mov l,a		; nejprve L := X/2 (CY=zbytek)
	mvi h,tbPix2Ofs/256
	mov a,m		; A := pixelov ofset 0..5
	ral
	sta MysBit
	inr h
	mov e,m
	mvi d,0
	pop h
	dad d
	shld MysAdr
	ret

; ****************************************************
; Nastaven citlivosti myi
; vstup: A=0: bez myi
;        A=1: mal citlivost    (verze emultor)
;	 A=2: stedn citlivost (std HW mouse 130DPI )
;        A=3: vysok citlivost  (homemade 40DPI)
; ****************************************************

MysDPISet
	lda MysSensitivity	; podle nastaven myi
	cpi 4			; (pouze platnho)
	rnc
	lxi h,tbMysDPISet	; z tabulky citlivost	
	rst 3			; vybere tu poadovanou
	shld MysSenstvtPos+1	; a ulo ji

	lda MysSensitivity	; podle nastaven myi
	ana a			; vybere
	lxi h,KbdOvladac	; bu ovlada klvesnice
	lxi d,000E6h		; a modifiktory procedury
	jz MysDPISetup		; pro uvolnn tlatek
	lxi h,MysDriver		; nebo ovlada myi
	lxi d,0C0F6h		; a modifiktory...
MysDPISetup
	shld Ovladani+1		; ulo ovlada my/klvesnice
	mov a,d			; a modifiktory...
	sta MouseKeyReleaseModA+1
	mov a,e
	sta MouseKeyReleaseModB
	sta MouseKeyReleaseModC
	ret

tbMysDPISet
	.dw MysDPIEmu,MysDPIEmu,MysDPIStd,MysDPIHom

; **************************************
; Tabulka druhch mocnin a jej velikost
; **************************************

tbRoot
	.db 0,1,4,9,16,25,36,49,64,81,100,121,144,169,196
	.db 225
Max_dL	.equ $-tbRoot-1

; ***********************************
; nvrat z obsluhy jednotlivch smr
; a klovn prstk dX a dY 
; ***********************************

MysQuit
	mov a,b		; C = ABS(X)
	ana a		; +uschovat info o zmn znamnka
	push psw
	cm Neg_BC

	mov a,d		; E = ABS(Y)
	ana a		; +uschovat info o zmn znamnka
	push psw
	cm Neg_DE

	lxi h,tbRoot	; (HL) = dX^2
	mvi a,Max_dL
	cmp c
	jc MysMax_dX
	mov a,c
MysMax_dX
	rst 2
	push h

	lxi h,tbRoot	; ACC = dY^2
	mvi a,Max_dL
	cmp e
	jc MysMax_dY
	mov a,e
MysMax_dY
	rst 2
	mov a,m

	pop h		; A = dX^2 + dY^2 (<256)
	add m
	jnc MysSenstvtPos
	mvi a,255	; max dL

MysSenstvtPos
	call MysDPIEmu	; vlastn dynamick pohyb

	mov a,b		; byla zmna souadnic bhem cyklu?
	ora c		; test dX
	ora d		; a dY
	ora e
	sta MysXCHG	; ACC=0 <=> my bez pohybu

	pop psw		; obnova znamnka dX a dY
	cm Neg_DE
	pop psw
	cm Neg_BC

	lhld MysY	; Y:=Y+dY (plus test pozice Y)
	dad d
	call TestYCoor	; test nastavench doraz v ose Y
	shld MysY	; uloit novou hodnotu souadnice Y

	lhld MysX	; X:=X+dX (plus test pozice X)
	dad b
	call TestXCoor	; test nastavench doraz v ose X
	shld MysX	; uloit novou hodnotu souadnice Y

	in mysport	; nast signly myi
	add a		; CY=prav tlatko, P=lev tlatko
	ret

; **************
; Ovlada SLOW
; vstup: BC = dX
;        DE = dY
; **************

MysDPIEmu
	cpi 255		; pro pomal pohyb
	jc MysDiv2	; f(dX,dY)=1/2

			; jinak f(dX,dY)=3/4
	call MysMul3	; x3

	ana a		; /2
	mov a,d
	rar
	mov d,a
	mov a,e
	rar
	mov e,a
	ana a
	mov a,b
	rar
	mov b,a
	mov a,c
	rar
	mov c,a

MysDiv2
	ana a		; /2
	mov a,d
	rar
	mov d,a
	mov a,e
	rar
	mov e,a
	ana a
	mov a,b
	rar
	mov b,a
	mov a,c
	rar
	mov c,a
	ret

; **************
; Ovlada NORM
; vstup: BC = dX
;        DE = dY
; **************

MysDPIStd
	cpi 64		; pro pomal pohyb
	rc		; f(dX,dY)=1

			; jinak f(dX,dY)=3/2
	call MysMul3	; x3

	jmp MysDiv2	; /2

; **************
; Ovlada FAST
; vstup: BC = dX
;        DE = dY
; **************

MysDPIHom
	cpi 4		; x1
	rc
	cpi 16		; x3
	jnc MysMul3

MysMul2
	mov a,e		; x2
	add a
	mov e,a
	mov a,d
	adc a
	mov d,a
	mov a,c
	add a
	mov c,a
	mov a,b
	adc a
	mov b,a
	ret
	
MysMul3
	mov l,e		; x3
	mov h,d
	dad h
	dad d
	mov d,h
	mov e,l
	mov l,c
	mov h,b
	dad h
	dad b
	mov b,h
	mov c,l
	ret

; *************************
; Zmna znamnka sla v BC
; *************************

Neg_BC	mov a,c
	cma
	mov c,a
	mov a,b
	cma
	mov b,a
	inx b
	ret

; *************************
; Zmna znamnka sla v DE
; *************************

Neg_DE	mov a,e
	cma
	mov e,a
	mov a,d
	cma
	mov d,a
	inx d
	ret
